@@ -33,19 +33,20 @@ starrydb module provide functions of a NoSQL database for lightweight data.
 import os
 import time
 import shutil
-import msgpack
-import stat
-import pylibmc
 import inspect
 import threading
 import logging
 import subprocess
 import functools
+import pathlib
 import multiprocessing.connection
 import concurrent.futures
 from contextlib import contextmanager
 
-VERSION = "0.2.0"
+import msgpack
+import pylibmc
+
+VERSION = "0.3.0"
 
 class StarryError(Exception):
     def __init__(self, output):
@@ -66,7 +67,7 @@ class StarryLog(object):
     def __init__(self, file=None, lvl=logging.NOTSET):
         self.logger = None
 
-        if os.path.isfile(file):
+        if pathlib.Path(file).is_file():
             try:
                 logging.basicConfig(
                     handlers=[logging.FileHandler(file)],
@@ -178,7 +179,7 @@ class LibMemcachedClient(pylibmc.ThreadMappedPool):
 
 class StarryServer(object):
     def __init__(self, database, address, cache_size=0, log_file=None, rw_ratio=5):
-        self.database = os.path.abspath(database)
+        self.database = pathlib.Path(database)
         self.address = address
         self.cache_size = cache_size
         self.cache = None
@@ -194,42 +195,43 @@ class StarryServer(object):
     def __del__(self):
         if self.cache is not None:
             try:
-                subprocess.check_output(f"fuser -k {os.path.join(self.database, 'cache.sock')}", shell=True, stderr=subprocess.STDOUT, universal_newlines=True)
+                subprocess.check_output(f"fuser -k {str(self.database.joinpath('cache.sock').resolve())}", shell=True, stderr=subprocess.STDOUT, universal_newlines=True)
             except:
                 pass
 
     def start(self, max_workers=512, backlog=512, authkey=b'starry'):
-        if not os.path.isdir(self.database):
-            os.mkdir(self.database)
+        self.database.mkdir(exist_ok=True)
 
         if type(self.address) is str:
-            _removeFile(self.address)
+            try:
+                os.remove(self.address)
+            except:
+                pass
 
         if int(self.cache_size) > 0:
             try:
-                subprocess.check_output(f"fuser -k {os.path.join(self.database, 'cache.sock')}", shell=True, stderr=subprocess.STDOUT, universal_newlines=True)
+                subprocess.check_output(f"fuser -k {str(self.database.joinpath('cache.sock').resolve())}", shell=True, stderr=subprocess.STDOUT, universal_newlines=True)
             except:
                 pass
 
             try:
-                os.system(f"memcached -d -u root -a 0777 -D : -C -m {int(self.cache_size)} -s {os.path.join(self.database, 'cache.sock')}")
+                os.system(f"memcached -d -u root -a 0777 -D : -C -m {int(self.cache_size)} -s {str(self.database.joinpath('cache.sock').resolve())}")
 
                 time.sleep(1)
 
-                self.cache = LibMemcachedClient(os.path.join(self.database, "cache.sock"))
+                self.cache = LibMemcachedClient(str(self.database.joinpath('cache.sock').resolve()))
             except:
                 try:
-                    subprocess.check_output(f"fuser -k {os.path.join(self.database, 'cache.sock')}", shell=True, stderr=subprocess.STDOUT, universal_newlines=True)
+                    subprocess.check_output(f"fuser -k {str(self.database.joinpath('cache.sock').resolve())}", shell=True, stderr=subprocess.STDOUT, universal_newlines=True)
                 except:
                     pass
 
                 self.cache = None
 
-        backup_dir = os.path.join(self.database, ".backup/")
+        if not self.database.joinpath(".backup").is_dir():
+            self.database.joinpath(".backup").mkdir(exist_ok=True)
+            self.database.joinpath(".backup", "~.lock").touch()
 
-        if not os.path.isdir(backup_dir):
-            os.mkdir(backup_dir)
-            os.mknod(os.path.join(backup_dir, "~.lock"))
             self.backup()
 
         with multiprocessing.connection.Listener(self.address, backlog=backlog, authkey=authkey) as listener:
@@ -446,7 +448,7 @@ class StarryServer(object):
         mark = {"mode": "r", "members": [None]}
 
         with self._lock(mark):
-            return (True, [elem.name.replace('.mpb', '') for elem in os.scandir(self.database) if elem.name.endswith(".mpb")])
+            return (True, [elem.stem for elem in self.database.glob("*") if elem.suffix == ".mpb"])
 
     def getAll(self):
         values = {}
@@ -454,12 +456,13 @@ class StarryServer(object):
         mark = {"mode": "r", "members": [None]}
 
         with self._lock(mark):
-            for key in [elem.name.replace('.mpb', '') for elem in os.scandir(self.database) if elem.name.endswith(".mpb")]:
-                (ret, value) = self._get([key])
-                if ret is False:
-                    return (ret, value)
+            for elem in self.database.glob("*"):
+                if elem.suffix == ".mpb":
+                    (ret, value) = self._get([elem.stem])
+                    if ret is False:
+                        return (ret, value)
 
-                values[key] = value
+                    values[elem.stem] = value
 
             return (True, values)
 
@@ -467,10 +470,8 @@ class StarryServer(object):
         mark = {"mode": "w", "members": [None]}
 
         with self._lock(mark):
-            keys = [elem.name.replace('.mpb', '') for elem in os.scandir(self.database) if elem.name.endswith(".mpb")]
-
             for key, value in values.items():
-                if key in keys:
+                if self.database.joinpath(f"{key}.mpb").exists():
                     (ret, msg) = self._update([key], value)
                     if ret is False:
                         return (ret, msg)
@@ -479,9 +480,10 @@ class StarryServer(object):
                     if ret is False:
                         return (ret, msg)
 
-            for key in keys:
-                if key not in values:
-                    self._delete([key])
+            for elem in self.database.glob("*"):
+                if elem.suffix == ".mpb":
+                    if elem.stem not in values:
+                        self._delete([elem.stem])
 
             return (True, "")
 
@@ -489,10 +491,11 @@ class StarryServer(object):
         mark = {"mode": "w", "members": [None]}
 
         with self._lock(mark):
-            for key in [elem.name.replace('.mpb', '') for elem in os.scandir(self.database) if elem.name.endswith(".mpb")]:
-                (ret, msg) = self._backup(key)
-                if ret is False:
-                    return (ret, msg)
+            for elem in self.database.glob("*"):
+                if elem.suffix == ".mpb":
+                    (ret, msg) = self._backup(elem.stem)
+                    if ret is False:
+                        return (ret, msg)
 
             return (True, "")
 
@@ -500,120 +503,121 @@ class StarryServer(object):
         mark = {"mode": "w", "members": [None]}
 
         with self._lock(mark):
-            keys = [elem.name.replace('.mpb', '') for elem in os.scandir(self.database) if elem.name.endswith(".mpb")]
-            backup_keys = [elem.name.replace('.mpb', '') for elem in os.scandir(f"{self.database}/.backup") if elem.name.endswith(".mpb")]
+            for elem in self.database.glob("*"):
+                if elem.suffix == ".mpb":
+                    (ret, msg) = self._get([elem.stem])
+                    if ret is False:
+                        self._recover(elem.stem)
 
-            for key in backup_keys:
-                if key not in keys:
-                    keys.append(key)
-
-            for key in keys:
-                (ret, msg) = self._get([key])
-                if ret is False:
-                    self._recover(key)
+            for elem in self.database.joinpath(".backup").glob("*"):
+                if elem.suffix == ".mpb" and not self.database.joinpath(elem.name).exists():
+                    (ret, msg) = self._get([elem.stem])
+                    if ret is False:
+                        self._recover(elem.stem)
 
             return (True, "")
 
     def save(self, path):
-        path = os.path.abspath(path)
+        path = pathlib.Path(path)
 
-        if not os.path.isdir(path):
-            self.logger.error(f"No {path}.")
-            return (False, f"No {path}.")
+        if not path.is_dir():
+            self.logger.error(f"No {str(path)}.")
+            return (False, f"No {str(path)}.")
 
-        for elem in os.scandir(path):
-            if elem.name.endswith(".mpb"):
-                _removeFile(elem.path)
-                _removeFile(f"{elem.path}.tmp")
+        for elem in path.glob("*"):
+            if elem.suffix == ".mpb":
+                elem.unlink()
+            elif elem.suffix == ".tmp":
+                elem.unlink()
 
         mark = {"mode": "w", "members": [None]}
 
         with self._lock(mark):
-            for key in [elem.name.replace('.mpb', '') for elem in os.scandir(self.database) if elem.name.endswith(".mpb")]:
-                (ret, msg) = self._backup(key, path=path)
-                if ret is False:
-                    return (ret, msg)
+            for elem in self.database.glob("*"):
+                if elem.suffix == ".mpb":
+                    (ret, msg) = self._backup(elem.stem, path=path)
+                    if ret is False:
+                        return (ret, msg)
 
             return (True, "")
 
     def load(self, path):
-        path = os.path.abspath(path)
+        path = pathlib.Path(path)
 
-        if not os.path.isdir(path):
-            self.logger.error(f"No {path}.")
-            return (False, f"No {path}.")
-
-        files = [elem.name.replace('.mpb', '') for elem in os.scandir(path) if elem.name.endswith(".mpb")]
+        if not path.is_dir():
+            self.logger.error(f"No {str(path)}.")
+            return (False, f"No {str(path)}.")
 
         mark = {"mode": "w", "members": [None]}
 
         with self._lock(mark):
-            for key in files:
-                (ret, msg) = self._load(key, path)
-                if ret is False:
-                    return (ret, msg)
+            for elem in path.glob("*"):
+                if elem.suffix == ".mpb":
+                    (ret, msg) = self._load(elem.stem, path)
+                    if ret is False:
+                        return (ret, msg)
 
-            for key in [elem.name.replace('.mpb', '') for elem in os.scandir(self.database) if elem.name.endswith(".mpb")]:
-                if key not in files:
-                    self._delete([key])
+            for elem in self.database.glob("*"):
+                if elem.suffix == ".mpb" and not path.joinpath(elem.name).exists():
+                    self._delete([elem.stem])
 
             return (True, "")
 
     def replace(self, path):
-        path = os.path.abspath(path)
+        path = pathlib.Path(path)
 
-        if not os.path.isdir(path):
-            self.logger.error(f"No {path}.")
-            return (False, f"No {path}.")
-
-        files = [elem.name.replace('.mpb', '') for elem in os.scandir(path) if elem.name.endswith(".mpb")]
+        if not path.is_dir():
+            self.logger.error(f"No {str(path)}.")
+            return (False, f"No {str(path)}.")
 
         mark = {"mode": "w", "members": [None]}
 
         with self._lock(mark):
-            for key in files:
-                (ret, msg) = self._load(key, path)
-                if ret is False:
-                    return (ret, msg)
+            for elem in path.glob("*"):
+                if elem.suffix == ".mpb":
+                    (ret, msg) = self._load(elem.stem, path)
+                    if ret is False:
+                        return (ret, msg)
 
-            for key in [elem.name.replace('.mpb', '') for elem in os.scandir(self.database) if elem.name.endswith(".mpb")]:
-                if key not in files:
-                    self._delete([key])
+            for elem in self.database.glob("*"):
+                if elem.suffix == ".mpb" and not path.joinpath(elem.name).exists():
+                    self._delete([elem.stem])
 
-            for key in files:
-                _removeFile(os.path.join(path, f"{key}.mpb"))
+            for elem in path.glob("*"):
+                if elem.suffix == ".mpb":
+                    elem.unlink()
 
             return (True, "")
 
     def upgrade(self, path):
-        path = os.path.abspath(path)
+        path = pathlib.Path(path)
 
-        if not os.path.isdir(path):
-            self.logger.error(f"No {path}.")
-            return (False, f"No {path}.")
-
-        files = [elem.name.replace('.mpb', '') for elem in os.scandir(path) if elem.name.endswith(".mpb")]
+        if not path.is_dir():
+            self.logger.error(f"No {str(path)}.")
+            return (False, f"No {str(path)}.")
 
         mark = {"mode": "w", "members": [None]}
 
         with self._lock(mark):
-            for key in files:
-                (ret, msg) = self._load(key, path)
-                if ret is False:
-                    return (ret, msg)
+            for elem in path.glob("*"):
+                if elem.suffix == ".mpb":
+                    (ret, msg) = self._load(elem.stem, path)
+                    if ret is False:
+                        return (ret, msg)
 
             return (True, "")
 
     def clear(self):
-        for elem in os.scandir(self.database):
+        for elem in self.database.glob("*"):
             if elem.is_file():
-                (ret, msg) = _removeFile(elem.path)
-                if ret is False:
-                    self.logger.error(msg)
-                    return (ret, msg)
+                try:
+                    elem.unlink()
+                except Exception as e:
+                    self.logger.error(f"Caught exception: {e.__class__}({e})")
+                    return (False, f"Caught exception: {e.__class__}({e})")
             elif elem.isdir():
                 try:
-                    shutil.rmtree(elem.path)
+                    shutil.rmtree(elem)
                 except Exception as e:
                     self.logger.error(f"Caught exception: {e.__class__}({e})")
                     return (False, f"Caught exception: {e.__class__}({e})")
@@ -629,7 +633,7 @@ class StarryServer(object):
         return (True, "")
 
     def destroy(self):
-        if os.path.isdir(self.database):
+        if self.database.is_dir():
             try:
                 shutil.rmtree(self.database)
             except Exception as e:
@@ -694,11 +698,9 @@ class StarryServer(object):
             if type(filters) is not list:
                 filters = [filters]
 
-        file = os.path.join(self.database, f"{key}.mpb")
-
         value = self._getCache(key)
         if value is None:
-            (ret, data) = _readFile(file)
+            (ret, data) = _readFile(self.database.joinpath(f"{key}.mpb"))
             if ret is False:
                 return (ret, data)
 
@@ -720,25 +722,28 @@ class StarryServer(object):
     def _update(self, route, part):
         branch = route[:]
         key = branch.pop(0)
-        file = os.path.join(self.database, f"{key}.mpb")
-        backup = os.path.join(self.database, ".backup/", f"{key}.mpb")
+        file = self.database.joinpath(f"{key}.mpb")
 
         if len(branch) == 0:
-            if not os.path.exists(file):
+            if not file.exists():
                 try:
-                    os.mknod(file)
+                    file.touch()
                 except Exception as e:
-                    _removeFile(file)
+                    try:
+                        file.unlink()
+                    except:
+                        pass
+
                     return (False, f"Caught exception: {e.__class__}({e})")
             elif self._getCache(key) == part:
                 return (True, "")
 
             value = part
         else:
-            if not os.path.exists(file):
+            if not file.exists():
                 return (False, f"No {key}.")
 
-            if not os.path.exists(backup):
+            if not self.database.joinpath(".backup", f"{key}.mpb").exists():
                 (ret, msg) = self._backup(key)
                 if ret is False:
                     return (ret, msg)
@@ -776,16 +781,15 @@ class StarryServer(object):
     def _insert(self, route, part):
         branch = route[:]
         key = branch.pop(0)
-        file = os.path.join(self.database, f"{key}.mpb")
-        backup = os.path.join(self.database, ".backup/", f"{key}.mpb")
+        file = self.database.joinpath(f"{key}.mpb")
 
         if len(branch) == 0:
             value = part
         else:
-            if not os.path.exists(file):
+            if not file.exists():
                 return (False, f"No {key}.")
 
-            if not os.path.exists(backup):
+            if not self.database.joinpath(".backup", f"{key}.mpb").exists():
                 (ret, msg) = self._backup(key)
                 if ret is False:
                     return (ret, msg)
@@ -811,11 +815,15 @@ class StarryServer(object):
         except Exception as e:
             return (False, f"Caught exception: {e.__class__}({e})")
 
-        if not os.path.exists(file):
+        if not file.exists():
             try:
-                os.mknod(file)
+                file.touch()
             except Exception as e:
-                _removeFile(file)
+                try:
+                    file.unlink()
+                except:
+                    pass
+
                 return (False, f"Caught exception: {e.__class__}({e})")
 
         (ret, msg) = _writeFile(file, value)
@@ -830,23 +838,34 @@ class StarryServer(object):
     def _delete(self, route):
         branch = route[:]
         key = branch.pop(0)
-        file = os.path.join(self.database, f"{key}.mpb")
-        backup = os.path.join(self.database, ".backup/", f"{key}.mpb")
+        file = self.database.joinpath(f"{key}.mpb")
+        backup_file = self.database.joinpath(".backup", f"{key}.mpb")
 
-        if not os.path.exists(file):
+        if not file.exists():
             self._deleteCache(key)
 
-            _removeFile(f"{file}.tmp")
+            try:
+                file.with_suffix(".mpb.tmp").unlink()
+            except:
+                pass
 
             if len(branch) == 0:
-                _removeFile(backup)
-                _removeFile(f"{backup}.tmp")
+                try:
+                    backup_file.unlink()
+                except:
+                    pass
+
+                try:
+                    backup_file.with_suffix(".mpb.tmp").unlink()
+                except:
+                    pass
+
                 return (True, "")
             else:
                 self._recover(key)
                 return (False, f"No {key}.")
 
-        if not os.path.exists(backup):
+        if not backup_file.exists():
             (ret, msg) = self._backup(key)
             if ret is False:
                 return (ret, msg)
@@ -854,15 +873,26 @@ class StarryServer(object):
         if len(branch) == 0:
             self._deleteCache(key)
 
-            _removeFile(f"{file}.tmp")
+            try:
+                file.with_suffix(".mpb.tmp").unlink()
+            except:
+                pass
 
-            (ret, msg) = _removeFile(file)
-            if ret is False:
+            try:
+                file.unlink()
+            except Exception as e:
                 self._recover(key)
-                return (False, msg)
+                return (False, f"Caught exception: {e.__class__}({e})")
 
-            _removeFile(backup)
-            _removeFile(f"{backup}.tmp")
+            try:
+                backup_file.unlink()
+            except:
+                pass
+
+            try:
+                backup_file.with_suffix(".mpb.tmp").unlink()
+            except:
+                pass
 
             return (True, "")
 
@@ -895,14 +925,9 @@ class StarryServer(object):
         return self._backup(key, check_flag=False)
 
     def _backup(self, key, check_flag=True, path=None):
-        file = os.path.join(self.database, f"{key}.mpb")
+        file = self.database.joinpath(f"{key}.mpb")
 
-        if path is None:
-            backup = os.path.join(self.database, ".backup/", f"{key}.mpb")
-        else:
-            backup = os.path.join(path, f"{key}.mpb")
-
-        if not os.path.exists(file):
+        if not file.exists():
             return (False, f"No {key}.")
 
         if check_flag is True:
@@ -916,29 +941,39 @@ class StarryServer(object):
                 self._recover(key)
                 return (False, f"Caught exception: {e.__class__}({e})")
 
-        return _copyFile(file, backup, statement=not os.path.exists(backup))
+        if path is None:
+            path = self.database.joinpath(".backup")
+
+        return _copyFile(file, path.joinpath(f"{key}.mpb"))
 
     def _recover(self, key):
-        file = os.path.join(self.database, f"{key}.mpb")
-        backup = os.path.join(self.database, ".backup/", f"{key}.mpb")
+        backup_file = self.database.joinpath(".backup", f"{key}.mpb")
 
         self._deleteCache(key)
 
-        if not os.path.exists(backup):
-            return (False, f"No {backup}.")
+        if not backup_file.exists():
+            return (False, f"No {str(backup_file)}.")
 
-        (ret, value) = _readFile(backup)
+        (ret, value) = _readFile(backup_file)
         if ret is False:
             return (ret, value)
 
         try:
             data = msgpack.unpackb(value, strict_map_key=False)
         except Exception as e:
-            _removeFile(backup)
-            _removeFile(f"{backup}.tmp")
+            try:
+                backup_file.unlink()
+            except:
+                pass
+
+            try:
+                backup_file.with_suffix(".mpb.tmp").unlink()
+            except:
+                pass
+
             return (False, f"Caught exception: {e.__class__}({e})")
 
-        (ret, msg) = _copyFile(backup, file, statement=not os.path.exists(file))
+        (ret, msg) = _copyFile(backup_file, self.database.joinpath(f"{key}.mpb"))
         if ret is False:
             return (ret, msg)
 
@@ -947,14 +982,14 @@ class StarryServer(object):
         return (True, data)
 
     def _load(self, key, path):
-        file = os.path.join(self.database, f"{key}.mpb")
-        source = os.path.join(path, f"{key}.mpb")
-        backup = os.path.join(self.database, ".backup/", f"{key}.mpb")
+        file = self.database.joinpath(f"{key}.mpb")
+        backup_file = self.database.joinpath(".backup", f"{key}.mpb")
+        source_file = path.joinpath(f"{key}.mpb")
 
-        if not os.path.exists(source):
-            return (False, f"No {source}.")
+        if not source_file.exists():
+            return (False, f"No {str(source_file)}.")
 
-        (ret, value) = _readFile(source)
+        (ret, value) = _readFile(source_file)
         if ret is False:
             return (ret, value)
 
@@ -963,18 +998,29 @@ class StarryServer(object):
         except Exception as e:
             return (False, f"Caught exception: {e.__class__}({e})")
 
-        if os.path.exists(file):
-            if not os.path.exists(backup):
+        if file.exists():
+            if not backup_file.exists():
                 (ret, msg) = self._backup(key)
                 if ret is False:
                     return (ret, msg)
 
-        (ret, msg) = _copyFile(source, file, statement=not os.path.exists(file))
+        (ret, msg) = _copyFile(source_file, file)
         if ret is False:
-            if not os.path.exists(backup):
-                _removeFile(file)
-                _removeFile(f"{file}.tmp")
-                _removeFile(f"{backup}.tmp")
+            if not backup_file.exists():
+                try:
+                    file.unlink()
+                except:
+                    pass
+
+                try:
+                    file.with_suffix(".mpb.tmp").unlink()
+                except:
+                    pass
+
+                try:
+                    backup_file.with_suffix(".mpb.tmp").unlink()
+                except:
+                    pass
             else:
                 self._recover(key)
 
@@ -1303,78 +1349,67 @@ def _deletePart(value, branch):
 
 def _readFile(src):
     try:
-        with open(src, "rb") as descriptor:
-            value = descriptor.read()
+        return (True, src.read_bytes())
     except Exception as e:
         return (False, f"Caught exception: {e.__class__}({e})")
 
-    return (True, value)
-
 def _writeFile(dst, value):
     try:
-        with open(dst, "wb", buffering=0) as descriptor:
+        with dst.open(mode="wb", buffering=0) as descriptor:
             descriptor.write(value)
             descriptor.truncate()
 
             descriptor.flush()
             os.fdatasync(descriptor.fileno())
 
-        if len(value) != os.path.getsize(dst):
+        if len(value) != dst.stat().st_size:
             return (False, "No space left on device.")
     except Exception as e:
         return (False, f"Caught exception: {e.__class__}({e})")
 
     return (True, "")
 
-def _removeFile(file):
-    try:
-        if os.path.exists(file):
-            os.remove(file)
-    except Exception as e:
-        return (False, f"Caught exception: {e.__class__}({e})")
-
-    return (True, "")
-
-def _copyFile(src, dst, statement=False):
+def _copyFile(src, dst):
     (ret, value) = _readFile(src)
     if ret is False:
         return (ret, value)
 
-    if statement is True:
-        try:
-            st = os.stat(src)
-        except:
-            statement = False
+    if dst.exists():
+        tmp_path = dst.with_suffix(".mpb.tmp")
 
-    tmp_file = None
-
-    if os.path.exists(dst):
-        tmp_file = f"{dst}.tmp"
-    else:
-        (ret, msg) = _writeFile(dst, value)
+        (ret, msg) = _writeFile(tmp_path, value)
         if ret is False:
-            _removeFile(dst)
-            return (ret, msg)
+            try:
+                tmp_path.unlink()
+            except:
+                pass
 
-    if tmp_file is not None:
-        (ret, msg) = _writeFile(tmp_file, value)
-        if ret is False:
-            _removeFile(tmp_file)
             return (ret, msg)
 
         try:
-            os.rename(tmp_file, dst)
+            tmp_path.rename(dst)
         except:
-            _removeFile(tmp_file)
+            try:
+                tmp_path.unlink()
+            except:
+                pass
 
             (ret, msg) = _writeFile(dst, value)
             if ret is False:
                 return (ret, msg)
+    else:
+        (ret, msg) = _writeFile(dst, value)
+        if ret is False:
+            try:
+                dst.unlink()
+            except:
+                pass
+
+            return (ret, msg)
 
-    if statement is True:
         try:
-            os.chmod(dst, stat.S_IMODE(st.st_mode))
-            os.chown(dst, st.st_uid, st.st_gid)
+            dst.chmod(src.stat().st_mode)
+            shutil.chown(dst, user=src.stat().st_uid, group=src.stat().st_gid)
         except:
             pass
 
